<?php 

/**
 Multishairport Administration Controller

 @Copyright 2016 Stefan Rick
 @author Stefan Rick
 Mail: stefan@rick-software.de
 Web: http://www.max2play.com

 */

class Setup extends Service {	
	public $soundDevices = array();
	protected $pname = 'multishairport';
	public $soundDeviceLog = '';
	public $player = array();
	public $scriptPath = '';
	
	public function __construct(){								
		parent::__construct();
		$this->pluginname = _('Multishairport');
		
		$this->scriptPath = dirname(__FILE__).'/../scripts/';
		$this->registerLocale(dirname(__FILE__).'/../locale', 'multishairport');
		
		if($this->checkLicense(true) == false)
			return true;
		
		$this->getSoundDevices();
		
		$this->getPlayer();
		
		if(isset($_REQUEST['action'])){
			if(strpos($_REQUEST['action'], 'start_') !== FALSE){			
				$pos = substr($_REQUEST['action'], 6, 1);
				$this->player[$pos]->start();
			}
			
			if(strpos($_REQUEST['action'], 'stop_') !== FALSE){
				$pos = substr($_REQUEST['action'], 5, 1);
				$this->player[$pos]->stop();
			}			
			
			if(strpos($_REQUEST['action'], 'kill_') !== FALSE){
				$pos = substr($_REQUEST['action'], 5, 1);
				$this->player[$pos]->kill();
			}
			
			if(strpos($_REQUEST['action'], 'save_') === 0){
				$pos = substr($_REQUEST['action'], 5, 1);
				$this->player[$pos]->save();				
			}
			
			if($_REQUEST['action'] == 'autostartsave'){
				$this->selectAutostart(isset($_REQUEST['autostartmultishairport']) ? 1 : 0, false);
				//Add to Autostartconfig for autostart if a player is not startet at boot
				$this->selectAutostart(isset($_REQUEST['autostartmultishairport']) ? 1 : 0, true);
			}
			
			if($_REQUEST['action'] == 'playercountsave'){
				$this->savePlayerCount();
				$this->player = array();
				$this->getPlayer();
			}
			if($_REQUEST['action'] == 'setAudioOutputPI'){
				$this->setAudioOutputPI($_REQUEST['AudioOutputPI']);
			}
		}
		
		$this->view->autostartmultishairport = $this->checkAutostart('multishairport', false);
		
		$this->getAudioOutputPI();
		
		$this->getAllLogs();		
		
		//get all Messages from all Players
		foreach($this->player as $player){
			$this->view->message = array_merge($this->view->message, $player->view->message);
		}
		
		$this->showHelpSidebar();
	}		
	
	/**
	 * get Playable Sounddevices for Squeezelite / Shairport from Squeezelite
	 * Style:
	 * default:CARD=ALSA              - bcm2835 ALSA, bcm2835 ALSA - Default Audio Device
	 */
	public function getSoundDevices(){
		$this->soundDeviceLog = $this->writeDynamicScript(array('/opt/squeezelite/squeezelite -l'));
		//Regex zerlegung: 
		// $matches[1][x] = Name
		// $matches[2][x] = Karte/Device
		// $matches[3][x] = Beschreibung
		// $matches[4][x] = Name ist PCM Software Device
		// $matches[5][x] = Beschreibung von PCM Software Device
		// $matches[6][x] = PCM Software Device ohne Beschreibung
		preg_match_all('=^[ ]*([^:\n]+)[ ]*:[ ]*([^-\n ]+)[ ]*-[ ]*(.*)[ ]*|^[ ]*([^\n- ]+)[ ]*-[ ]*(.*)[ ]*|^[ ]*([^\n-:]+)[ ]*$=im', $this->soundDeviceLog, $matches);
		$count = count($matches[1]);
		for($i = 0; $i < $count; $i++){
			$dev = array();
			//Fallunterscheidung je nach Devices (Software / Hardware / mit und ohne Hinweistext)
			if($matches[1][$i] !== ''){
				$dev['name'] = $matches[1][$i];
				$dev['card'] = $matches[2][$i];			
				$dev['description'] = $matches[3][$i];
				$this->view->soundDevices[$matches[1][$i].':'.$matches[2][$i]] = $dev;
			}elseif($matches[4][$i] !== ''){
				$dev['name'] = $matches[4][$i]; 
				$dev['card'] = '';
				$dev['description'] = $matches[5][$i];
				//add without "plug:"
				if($matches[4][$i] == 'bluetoothspeaker' || $matches[4][$i] == 'pulse')
					$this->view->soundDevices[$matches[4][$i]] = $dev;
				else
					$this->view->soundDevices['plug:'.$matches[4][$i]] = $dev;
			}elseif($matches[6][$i] !== ''){
				$dev['name'] = $matches[6][$i];
				$dev['card'] = '';
				$dev['description'] = $matches[6][$i];
				$this->view->soundDevices['plug:'.$matches[6][$i]] = $dev;
			}
		}
		unset($this->view->soundDevices['plug:null']);
		
		//COPY Script to /etc/init.d and set execute rights for all users
		$lastupdate = 1459250422; // 29.03.2016
		if(!file_exists('/etc/init.d/multishairport') || filemtime ('/etc/init.d/multishairport') < $lastupdate){
			$this->writeDynamicScript(array('cp '.$this->scriptPath.'multishairport /etc/init.d/multishairport; chmod 755 /etc/init.d/multishairport'));
			$this->view->message[] = _t('Added Multishairport Startscript');
		}
		
		return true;
	}
	
	public function getPlayer(){
		$this->view->playercount = $this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'MULTISHAIRPORT_PLAYERCOUNT');
		for($i=0; $i < $this->view->playercount; $i++){
			$this->player[$i] = new Squeezeplayer($this->view->soundDevices, $i);
		}

		return true;
	}
	
	public function savePlayerCount(){
		if($this->view->playercount != intval($_REQUEST['playercount'])){
			$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'MULTISHAIRPORT_PLAYERCOUNT', intval($_REQUEST['playercount']));
			$this->view->message[] = _t('Number of players changed');
		}		
		return true;
	}		
	
	private function getAllLogs(){
		$out['SHAIRPORT VERSION'] = shell_exec('/opt/shairport/shairport -V');		
		$out['AUDIOPLAYER CONFIG'] = shell_exec('cat /opt/max2play/audioplayer.conf');		
		$out['OUTPUT SOUND DETAILS CARD 0'] = shell_exec('cat /proc/asound/card0/pcm0p/sub0/hw_params');
		$out['OUTPUT SOUND DETAILS CARD 1'] = shell_exec('cat /proc/asound/card1/pcm0p/sub0/hw_params');
		if($this->debug === TRUE){
			$out['SOUNDDEVICES IN USE'] = $this->writeDynamicScript(array('/bin/fuser -v /dev/snd/* 2>&1'));
		}
		$this->view->debug = $out;
		return true;
	}	
	
	/**
	 * Function for Raspberry PI to switch Output between Default, Jack and HDMI
	 * Default: if hdmi is connected it will be choosen	
	 */
	private function setAudioOutputPI($value = 0){
		if($this->getHardwareInfo() == 'Raspberry PI'){
			$this->view->audioOutputPI = $this->writeDynamicScript(array('sudo -u pi amixer cset numid=3 '.$value));
			$this->view->message[] = str_replace('$VALUE',$value, _('Raspberry PI Audio Output set to $VALUE'));
		}
		return true;
	}
	
	/**
	 * Function for Raspberry PI to get Audio-Output 
	 * can be Default=0, Headphone-Jack=1 or HDMI=2
	 */
	private function getAudioOutputPI(){
		if($this->getHardwareInfo() == 'Raspberry PI'){
			$this->view->audioOutputPI = trim($this->writeDynamicScript(array('su -l pi -c \'amixer -c ALSA cget numid=3 | grep ": values=" | sed "s/.*values=//"\'')), "\n");
			return $this->view->audioOutputPI;
		}else
			return false;
	}
	
	public function showHelpSidebar(){
		global $helpSidebar;
		$helpSidebar['title'] = _t('Help - Multishairport');
		$helpSidebar['content'] = _t('<ul><li>Make sure to remove the autostart from all players on the audioplayer tab, as they block the audiocard. Shairport will still be indicated as running on the audioplayer-tab once Multishairport is running (just ignore this).</li><li>Choose a different soundcard for each Shairport-Instance!</li></ul>');
		return true;
	}
		
}
include_once('Shairport.php');

$setup = new Setup();
  
include_once(dirname(__FILE__).'/../view/setup.php');