<?php 

/**
 Squeezelite Administration Controller

 @Copyright 2014 Stefan Rick
 @author Stefan Rick
 Mail: stefan@rick-software.de
 Web: http://www.netzberater.de

 */

class Squeezeplayer extends Service {
	protected $pname = 'shairport';
	public $viewname = 'Shairport';
	public $soundDevices = array();
	
	public function __construct($soundDevices, $instance = 0){								
		parent::__construct();			
		
		$this->view->soundDevices = $soundDevices;
		$this->instance = $instance;				
		
		$this->getShairportCommandline();
		
		$this->view->pid = $this->status();
			
	}		
	
	/**
	 * Read PID from /opt/squeezelite/log/squeezelite_INSTANCE.pid and get running process
	 */
	public function status($name = ''){
		if(file_exists('/opt/shairport/log/shairport_'.$this->instance.'.pid')){
			$pid = file_get_contents('/opt/shairport/log/shairport_'.$this->instance.'.pid');
			$shellanswer = shell_exec('ps -Al | grep "'.$pid.'.*'.$this->pname.'"');			
			if($shellanswer != ''){
				$this->view->pid = $pid;
				return $pid;				
			}
		}
		$this->view->pid = false;
		return false;
	}
	
	public function start($name = '', $command = '', $statusname = '', $rootstart = false, $background = false){					
		if($this->view->pid !== FALSE){
			$this->view->message[] = _('Process is already running');
			return false;
		}
		
		$startcom = 'sudo su -l '.$this->getSystemUser().' -c "/etc/init.d/multishairport start '.$this->instance.'"';
				
		$this->writeDynamicScript(array($startcom));		
	
		sleep(2);
	
		$shellanswer = _('Trying to launch');
	
		$pid = $this->status();
	
		if($pid != FALSE){
			$shellanswer .= ' ... '._('successful with ID').' '.$pid;
		}else{
			$shellanswer .= ' ... '._('NOT successful');
		}
		$this->view->message[] = $shellanswer;
		return true;
	}
	
	public function stop($name = '', $command = '', $statusname = '', $rootstop = false){
		if($this->view->pid === FALSE){
			$this->view->message[] = _('Unable to stop process - it seems that it is not running');
			return false;
		}
	
		$stopcom = 'sudo su -l '.$this->getSystemUser().' -c "/etc/init.d/multishairport stop '.$this->instance.'"';
	
		$this->writeDynamicScript(array($stopcom));
	
		sleep(2);
	
		$shellanswer = _('Trying to stop');
	
		$pid = $this->status();
	
		if($pid === FALSE){
			$shellanswer .= ' ... '._('successfully stopped');
		}else{
			$shellanswer .= ' ... '._('could NOT stop process - try kill instead');
		}
		$this->view->message[] = $shellanswer;
		
		return true;
	}
	
	public function kill($name = ''){
		
		if($this->view->pid === FALSE){
			$this->view->message[] = _('Unable to kill process - it seems that it is not running');
			return false;
		}
	
		shell_exec("sudo kill -9 ".$this->view->pid);
	
		sleep(3);
	
		$shellanswer = _('Trying to stop');
		$pid = $this->status();
		if($pid === FALSE){
			$shellanswer .= ' ... '._('successfully killed');
		}else{
			$shellanswer .= ' ... '._('could not kill process');
		}
		$this->view->message[] = $shellanswer;
		
		return true;
	}	
	
	
	/**
	 * Save Command-Line Options from
	 * squeezelite_soundcard
	 * squeezelite_commandline
	 * squeezelite_playername
	 * squeezelite_alsaparam
	 */
	public function save(){				
		$this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'MULTISHAIRPORT_NAME_'.$this->instance, $_REQUEST['shairport_playername'][$this->instance]);
		
		$commandLine = array();
		$setsoundcard = $_REQUEST['shairport_soundcard'][$this->instance];
		if(in_array($setsoundcard, array_keys($this->view->soundDevices))){			
			$commandLine[] = '-d '.$setsoundcard;
		}else{
			$commandLine[] = '-d plug:dmixer';
		}		
		
		//TODO: Regex für korrekte Erkennung der Commandlineeingabe der Parameter
		$commandLine[] = trim($_REQUEST['shairport_commandline'][$this->instance]);
		
		$value = trim(implode(' ', $commandLine));
		if($this->saveConfigFileParameter('/opt/max2play/audioplayer.conf', 'MULTISHAIRPORT_PARAMETER_'.$this->instance, $value)){
			$this->view->message[] = str_replace('$SERVICE', $this->viewname ,_('Updated $SERVICE Settings - Restart $SERVICE to apply changes!'));
			
			if($this->status() !== FALSE){
				//Restart Service
				$this->stop();
				$this->start();
			}
		}				
		$this->getShairportCommandline();
		return true;
	}
	
	/**
	 * Get Commandline Options for Audioplayer by Config File
	 * @return boolean
	 */
	public function getShairportCommandline(){
		$output = $this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'MULTISHAIRPORT_PARAMETER_'.$this->instance);
		if(preg_match_all('=-d ([^ ]*)( (.*))?=', $output, $match)){
			$this->view->shairport_soundcard = trim($match[1][0]);			
			$this->view->shairport_commandline = $match[3][0];
		}
		$this->view->shairport_playername = $this->getConfigFileParameter('/opt/max2play/audioplayer.conf', 'MULTISHAIRPORT_NAME_'.$this->instance);
		if(!$this->view->shairport_playername){
			$this->view->shairport_playername = 'Max2Play_'.$this->instance;
		}
		return true;
	}				
}
